
package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.entity.BaseEntity;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.knowledge.entity.dao.Banner;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.dao.KnowledgeCategory;
import com.redxun.knowledge.entity.vo.BannerAdminVo;
import com.redxun.knowledge.entity.vo.BannerPcVo;
import com.redxun.knowledge.mapper.BannerMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* [业务--首页banner表]业务服务类
*/
@Slf4j
@Service
public class BannerServiceImpl extends SuperServiceImpl<BannerMapper, Banner> implements BaseService<Banner> {

    @Resource
    private BannerMapper bannerMapper;

    @Autowired
    private KnowledgeServiceImpl knowledgeService;

    @Autowired
    private KnowledgeCategoryServiceImpl knowledgeCategoryService;

    @Override
    public BaseDao<Banner> getRepository() {
        return bannerMapper;
    }


    /**
     * 后台查询全部banner展示(分页)
     * @param filter
     * @return
     */
    public IPage findAllBanner(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        IPage page = bannerMapper.findAllBanner(filter.getPage(), filter.getParams(), params);
        List<BannerAdminVo> bannerAdminVos = page.getRecords();
        bannerAdminVos.forEach(e -> {
            String knowledgeId = e.getKnowledgeId();
            if (!StringUtils.isEmpty(knowledgeId)){
                Map<String, Object> map = new HashMap<>();
                Knowledge knowledge = knowledgeService.get(knowledgeId);
                if (knowledge != null){
                    map.put("knowledgeName",knowledge.getTitle());
                }
                e.setParams(map);
            }
        });

        if (filter.getParams().containsKey("bannerId") && filter.getParams().containsKey("flag")){
            String bannerId = (String) filter.getParams().get("bannerId");
            Boolean flag = (Boolean) filter.getParams().get("flag");
            if (flag){
                Banner banner = this.get(bannerId);
                BannerAdminVo bannerAdminVo = new BannerAdminVo();
                BeanUtils.copyProperties(banner,bannerAdminVo);
                int i = bannerAdminVos.indexOf(bannerAdminVo);
                log.info("上升记录位置 -> {}",i);
                BannerAdminVo bannerAdminVoPre = bannerAdminVos.get(i - 1);
                // 交换排序
                int temp = bannerAdminVo.getSort();
                bannerAdminVo.setSort(bannerAdminVoPre.getSort());
                bannerAdminVoPre.setSort(temp);
            }
        }
        return page;
    }

    /**
     * 获取Banner详情
     * @param id
     * @return
     */
    @Override
    public Banner get(Serializable id) {
        Banner banner = this.getRepository().selectById(id);
        String knowledgeId = banner.getKnowledgeId();
        if (!StringUtils.isEmpty(knowledgeId)){
            Knowledge knowledge = knowledgeService.get(knowledgeId);
            if(knowledge != null){
                KnowledgeCategory knowledgeCategory = knowledgeCategoryService.findAllByCategoryId(knowledge.getCategoryId());
                Map<String,Object> map = new HashMap<>();
                map.put("knowledgeName",knowledge.getTitle());
                map.put("knowledgeCategory",knowledgeCategory);
                banner.setParams(map);
            }
        }
        return banner;
    }

    /**
     * 前台查询全部banner展示(默认展示10张)
     * @return
     */
    public List<BannerPcVo> getAllList() {
        QueryWrapper<Banner> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ENABLED",1);
        queryWrapper.orderByDesc("SORT").orderByDesc("CREATE_TIME_");
        List<Banner> bannersList = bannerMapper.selectList(queryWrapper);
        List<Banner> result = bannersList.parallelStream().limit(10).collect(Collectors.toList());
        List<BannerPcVo> collectBannerPcVo = result.parallelStream().map(e -> {
            BannerPcVo bannerPcVo = new BannerPcVo();
            BeanUtils.copyProperties(e, bannerPcVo);
            return bannerPcVo;
        }).collect(Collectors.toList());
        return collectBannerPcVo;
    }

    public void deleteBanner(String pkId) {
        bannerMapper.deleteBanner(pkId);
    }

    /**
     * 状态为true,上升;为false,下降
     * @param bannerId
     * @param status
     */
    public void goUpOrDownUp(String bannerId,Boolean status) {
        Banner banner = this.get(bannerId);
        Integer sort = banner.getSort();
        QueryWrapper<Banner> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("SORT",sort);
        List<Banner> banners = bannerMapper.selectList(queryWrapper);
        log.info("bannerList -> {}",banners);

    }
}
