
package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.entity.QueryData;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.base.search.QueryFilterBuilder;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.knowledge.entity.dao.News;
import com.redxun.knowledge.entity.vo.NewsAdminVo;
import com.redxun.knowledge.entity.vo.NewsPcVo;
import com.redxun.knowledge.mapper.NewsMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * [业务--新闻表]业务服务类
 */
@Slf4j
@Service
public class NewsServiceImpl extends SuperServiceImpl<NewsMapper, News> implements BaseService<News> {

    @Resource
    private NewsMapper newsMapper;

    @Override
    public BaseDao<News> getRepository() {
        return newsMapper;
    }

    /**
     * 后台新闻列表(分页)
     *
     * @param filter
     * @return
     */
    public IPage findAllNews(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        IPage page = newsMapper.findAllNews(filter.getPage(), filter.getParams(),params);
        List<News> newsList = page.getRecords();
        List<NewsAdminVo> collectNewsAdminVoList = newsList.parallelStream().map(e -> {
            NewsAdminVo newsAdminVo = new NewsAdminVo();
            BeanUtils.copyProperties(e, newsAdminVo);
            return newsAdminVo;
        }).collect(Collectors.toList());
        page.setRecords(collectNewsAdminVoList);
        return page;
    }

    /**
     * 首页新闻资讯 前台展示
     *
     * @return
     */
    public List<NewsPcVo> getAllList() {
        // 组装分页
        QueryData queryData = new QueryData();
        queryData.setPageNo(1);
        queryData.setPageSize(10);
        QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
        IPage page = this.findAllNews(filter);
        // 查询后台展示结果
        List<NewsAdminVo> records = page.getRecords();
        // 前台展示结果处理
        List<NewsPcVo> collectNewsPcVoList = records.stream().
                filter(e -> e.getEnabled() == 1).
                limit(6).
                map(e -> {
                    NewsPcVo newsPcVo = new NewsPcVo();
                    BeanUtils.copyProperties(e, newsPcVo);
                    return newsPcVo;
                }).collect(Collectors.toList());
        return collectNewsPcVoList;
    }

    /**
     * 前台分页列表
     *
     * @param filter
     * @return
     */
    public IPage findAllPcNews(QueryFilter filter) {
        IPage page = newsMapper.findAllPcNews(filter.getPage(), filter.getParams());
        return page;
    }

}
